package net.sergeych.crypto2

import kotlinx.datetime.Instant
import kotlinx.serialization.Serializable

interface VerifyingKey {
    val id: KeyId

    /**
     * Verify the signature and return true if it is correct.
     */
    fun verify(signature: UByteArray, message: UByteArray): Boolean
}

interface SigningKey {
    val verifyingKey: SigningPublicKey
    fun sign(message: UByteArray): UByteArray
    fun seal(message: UByteArray, expiresAt: Instant? = null): Seal
}

@Serializable
abstract class BinaryKeyBase() {

    abstract val keyBytes: UByteArray
    abstract val magick: KeysMagickNumber

    open val id by lazy { KeyId(magick, keyBytes) }

    override fun equals(other: Any?): Boolean {
        return other is BinaryKeyBase && other.keyBytes contentEquals keyBytes
    }

    override fun hashCode(): Int {
        return keyBytes.contentHashCode()
    }

    override fun toString(): String = keyBytes.encodeToBase64Url()

    companion object {

    }
}

open class IllegalSignatureException(text: String = "signed data is tampered or signature is corrupted") :
    IllegalStateException(text)

class ExpiredSignatureException(text: String) : IllegalSignatureException(text)