package net.sergeych.crypto2

import com.ionspin.kotlin.crypto.signature.InvalidSignatureException
import com.ionspin.kotlin.crypto.signature.Signature
import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

/**
 * Public key to verify signatures only
 */
@Serializable
class SigningPublicKey(override val keyBytes: UByteArray) : BinaryKeyBase(), VerifyingKey {
    /**
     * Verify the signature and return true if it is correct.
     */
    override fun verify(signature: UByteArray, message: UByteArray): Boolean = try {
        Signature.verifyDetached(signature, message, keyBytes)
        true
    } catch (_: InvalidSignatureException) {
        false
    }

    override fun toString(): String = "Pub:${super.toString()}"

    @Transient
    override val magick: KeysMagickNumber = KeysMagickNumber.defaultVerifying
}